@@ -12,10 +12,12 @@ import numpy as np
 import time
 import json
 import warnings
+import io
 
 from collections import deque
 from collections import OrderedDict
 from collections import Iterable
+from collections import defaultdict
 from .utils.generic_utils import Progbar
 from . import backend as K
 from .engine.training_utils import standardize_input_data
@@ -26,6 +28,11 @@ except ImportError:
     requests = None
 
 
+_TRAIN = 'train'
+_TEST = 'test'
+_PREDICT = 'predict'
+
+
 class CallbackList(object):
     """Container abstracting a list of callbacks.
 
@@ -39,106 +46,228 @@ class CallbackList(object):
         callbacks = callbacks or []
         self.callbacks = [c for c in callbacks]
         self.queue_length = queue_length
+        self.params = {}
+        self.model = None
+        self._reset_batch_timing()
+
+    def _reset_batch_timing(self):
+        self._delta_t_batch = 0.
+        self._delta_ts = defaultdict(lambda: deque([], maxlen=self.queue_length))
 
     def append(self, callback):
         self.callbacks.append(callback)
 
     def set_params(self, params):
+        self.params = params
         for callback in self.callbacks:
             callback.set_params(params)
 
     def set_model(self, model):
+        self.model = model
         for callback in self.callbacks:
             callback.set_model(model)
 
+    def _call_batch_hook(self, mode, hook, batch, logs=None):
+        """Helper function for all batch_{begin | end} methods."""
+        if not self.callbacks:
+            return
+        hook_name = 'on_{mode}_batch_{hook}'.format(mode=mode, hook=hook)
+        if hook == 'end':
+            if not hasattr(self, '_t_enter_batch'):
+                self._t_enter_batch = time.time()
+            # Batch is ending, calculate batch time
+            self._delta_t_batch = time.time() - self._t_enter_batch
+
+        logs = logs or {}
+        t_before_callbacks = time.time()
+        for callback in self.callbacks:
+            batch_hook = getattr(callback, hook_name)
+            batch_hook(batch, logs)
+        self._delta_ts[hook_name].append(time.time() - t_before_callbacks)
+
+        delta_t_median = np.median(self._delta_ts[hook_name])
+        if (self._delta_t_batch > 0. and
+           delta_t_median > 0.95 * self._delta_t_batch and
+           delta_t_median > 0.1):
+            warnings.warn(
+                'Method (%s) is slow compared '
+                'to the batch update (%f). Check your callbacks.'
+                % (hook_name, delta_t_median), RuntimeWarning)
+
+        if hook == 'begin':
+            self._t_enter_batch = time.time()
+
+    def _call_begin_hook(self, mode):
+        """Helper function for on_{train|test|predict}_begin methods."""
+        if mode == _TRAIN:
+            self.on_train_begin()
+        elif mode == _TEST:
+            self.on_test_begin()
+        else:
+            self.on_predict_begin()
+
+    def _call_end_hook(self, mode):
+        """Helper function for on_{train|test|predict}_end methods."""
+        if mode == _TRAIN:
+            self.on_train_end()
+        elif mode == _TEST:
+            self.on_test_end()
+        else:
+            self.on_predict_end()
+
+    def on_batch_begin(self, batch, logs=None):
+        self._call_batch_hook(_TRAIN, 'begin', batch, logs=logs)
+
+    def on_batch_end(self, batch, logs=None):
+        self._call_batch_hook(_TRAIN, 'end', batch, logs=logs)
+
     def on_epoch_begin(self, epoch, logs=None):
-        """Called at the start of an epoch.
+        """Calls the `on_epoch_begin` methods of its callbacks.
+
+        This function should only be called during train mode.
 
         # Arguments
             epoch: integer, index of epoch.
-            logs: dictionary of logs.
+            logs: dict, Currently no data is passed to this argument for this method
+                but that may change in the future.
         """
         logs = logs or {}
         for callback in self.callbacks:
             callback.on_epoch_begin(epoch, logs)
-        self._delta_t_batch = 0.
-        self._delta_ts_batch_begin = deque([], maxlen=self.queue_length)
-        self._delta_ts_batch_end = deque([], maxlen=self.queue_length)
+        self._reset_batch_timing()
 
     def on_epoch_end(self, epoch, logs=None):
-        """Called at the end of an epoch.
+        """Calls the `on_epoch_end` methods of its callbacks.
+
+        This function should only be called during train mode.
 
         # Arguments
             epoch: integer, index of epoch.
-            logs: dictionary of logs.
+            logs: dict, metric results for this training epoch, and for the
+                validation epoch if validation is performed. Validation result keys
+                are prefixed with `val_`.
         """
         logs = logs or {}
         for callback in self.callbacks:
             callback.on_epoch_end(epoch, logs)
 
-    def on_batch_begin(self, batch, logs=None):
-        """Called right before processing a batch.
+    def on_train_batch_begin(self, batch, logs=None):
+        """Calls the `on_train_batch_begin` methods of its callbacks.
 
         # Arguments
             batch: integer, index of batch within the current epoch.
-            logs: dictionary of logs.
+            logs: dict, has keys `batch` and `size` representing the current
+                batch number and the size of the batch.
         """
-        logs = logs or {}
-        t_before_callbacks = time.time()
-        for callback in self.callbacks:
-            callback.on_batch_begin(batch, logs)
-        self._delta_ts_batch_begin.append(time.time() - t_before_callbacks)
-        delta_t_median = np.median(self._delta_ts_batch_begin)
-        if (self._delta_t_batch > 0. and
-           delta_t_median > 0.95 * self._delta_t_batch and
-           delta_t_median > 0.1):
-            warnings.warn('Method on_batch_begin() is slow compared '
-                          'to the batch update (%f). Check your callbacks.'
-                          % delta_t_median)
-        self._t_enter_batch = time.time()
+        self._call_batch_hook(_TRAIN, 'begin', batch, logs=logs)
 
-    def on_batch_end(self, batch, logs=None):
-        """Called at the end of a batch.
+    def on_train_batch_end(self, batch, logs=None):
+        """Calls the `on_train_batch_end` methods of its callbacks.
 
         # Arguments
             batch: integer, index of batch within the current epoch.
-            logs: dictionary of logs.
+            logs: dict, metric results for this batch.
         """
-        logs = logs or {}
-        if not hasattr(self, '_t_enter_batch'):
-            self._t_enter_batch = time.time()
-        self._delta_t_batch = time.time() - self._t_enter_batch
-        t_before_callbacks = time.time()
-        for callback in self.callbacks:
-            callback.on_batch_end(batch, logs)
-        self._delta_ts_batch_end.append(time.time() - t_before_callbacks)
-        delta_t_median = np.median(self._delta_ts_batch_end)
-        if (self._delta_t_batch > 0. and
-           (delta_t_median > 0.95 * self._delta_t_batch and delta_t_median > 0.1)):
-            warnings.warn('Method on_batch_end() is slow compared '
-                          'to the batch update (%f). Check your callbacks.'
-                          % delta_t_median)
+        self._call_batch_hook(_TRAIN, 'end', batch, logs=logs)
+
+    def on_test_batch_begin(self, batch, logs=None):
+        """Calls the `on_test_batch_begin` methods of its callbacks.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, has keys `batch` and `size` representing the current
+                batch number and the size of the batch.
+        """
+        self._call_batch_hook(_TEST, 'begin', batch, logs=logs)
+
+    def on_test_batch_end(self, batch, logs=None):
+        """Calls the `on_test_batch_end` methods of its callbacks.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, metric results for this batch.
+        """
+        self._call_batch_hook(_TEST, 'end', batch, logs=logs)
+
+    def on_predict_batch_begin(self, batch, logs=None):
+        """Calls the `on_predict_batch_begin` methods of its callbacks.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, has keys `batch` and `size` representing the current
+                batch number and the size of the batch.
+        """
+        self._call_batch_hook(_PREDICT, 'begin', batch, logs=logs)
+
+    def on_predict_batch_end(self, batch, logs=None):
+        """Calls the `on_predict_batch_end` methods of its callbacks.
+
+        # Argument
+            batch: integer, index of batch within the current epoch.
+            logs: dict, metric results for this batch.
+        """
+        self._call_batch_hook(_PREDICT, 'end', batch, logs=logs)
 
     def on_train_begin(self, logs=None):
-        """Called at the beginning of training.
+        """Calls the `on_train_begin` methods of its callbacks.
 
         # Arguments
-            logs: dictionary of logs.
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
         """
-        logs = logs or {}
         for callback in self.callbacks:
             callback.on_train_begin(logs)
 
     def on_train_end(self, logs=None):
-        """Called at the end of training.
+        """Calls the `on_train_end` methods of its callbacks.
 
         # Arguments
-            logs: dictionary of logs.
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
         """
-        logs = logs or {}
         for callback in self.callbacks:
             callback.on_train_end(logs)
 
+    def on_test_begin(self, logs=None):
+        """Calls the `on_test_begin` methods of its callbacks.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+        for callback in self.callbacks:
+            callback.on_test_begin(logs)
+
+    def on_test_end(self, logs=None):
+        """Calls the `on_test_end` methods of its callbacks.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+        for callback in self.callbacks:
+            callback.on_test_end(logs)
+
+    def on_predict_begin(self, logs=None):
+        """Calls the `on_predict_begin` methods of its callbacks.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+        for callback in self.callbacks:
+            callback.on_predict_begin(logs)
+
+    def on_predict_end(self, logs=None):
+        """Calls the `on_predict_end` methods of its callbacks.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+        for callback in self.callbacks:
+            callback.on_predict_end(logs)
+
     def __iter__(self):
         return iter(self.callbacks)
 
@@ -180,23 +309,169 @@ class Callback(object):
     def set_model(self, model):
         self.model = model
 
-    def on_epoch_begin(self, epoch, logs=None):
-        pass
-
-    def on_epoch_end(self, epoch, logs=None):
-        pass
-
     def on_batch_begin(self, batch, logs=None):
-        pass
+        """A backwards compatibility alias for `on_train_batch_begin`."""
 
     def on_batch_end(self, batch, logs=None):
-        pass
+        """A backwards compatibility alias for `on_train_batch_end`."""
+
+    def on_epoch_begin(self, epoch, logs=None):
+        """Called at the start of an epoch.
+
+        Subclasses should override for any actions to run. This function should only
+        be called during train mode.
+
+        # Arguments
+            epoch: integer, index of epoch.
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+
+    def on_epoch_end(self, epoch, logs=None):
+        """Called at the end of an epoch.
+
+        Subclasses should override for any actions to run. This function should only
+        be called during train mode.
+
+        # Arguments
+            epoch: integer, index of epoch.
+            logs: dict, metric results for this training epoch, and for the
+                validation epoch if validation is performed. Validation result keys
+                are prefixed with `val_`.
+        """
+
+    def on_train_batch_begin(self, batch, logs=None):
+        """Called at the beginning of a training batch in `fit` methods.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, has keys `batch` and `size` representing the current
+                batch number and the size of the batch.
+        """
+        # For backwards compatibility
+        self.on_batch_begin(batch, logs=logs)
+
+    def on_train_batch_end(self, batch, logs=None):
+        """Called at the end of a training batch in `fit` methods.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, metric results for this batch.
+        """
+        # For backwards compatibility
+        self.on_batch_end(batch, logs=logs)
+
+    def on_test_batch_begin(self, batch, logs=None):
+        """Called at the beginning of a batch in `evaluate` methods.
+
+        Also called at the beginning of a validation batch in the `fit` methods,
+        if validation data is provided.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, has keys `batch` and `size` representing the current
+                batch number and the size of the batch.
+        """
+
+    def on_test_batch_end(self, batch, logs=None):
+        """Called at the end of a batch in `evaluate` methods.
+
+        Also called at the end of a validation batch in the `fit` methods,
+        if validation data is provided.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, metric results for this batch.
+        """
+
+    def on_predict_batch_begin(self, batch, logs=None):
+        """Called at the beginning of a batch in `predict` methods.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, has keys `batch` and `size` representing the current
+                batch number and the size of the batch.
+        """
+
+    def on_predict_batch_end(self, batch, logs=None):
+        """Called at the end of a batch in `predict` methods.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            batch: integer, index of batch within the current epoch.
+            logs: dict, metric results for this batch.
+        """
 
     def on_train_begin(self, logs=None):
-        pass
+        """Called at the beginning of training.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
 
     def on_train_end(self, logs=None):
-        pass
+        """Called at the end of training.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+
+    def on_test_begin(self, logs=None):
+        """Called at the beginning of evaluation or validation.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+
+    def on_test_end(self, logs=None):
+        """Called at the end of evaluation or validation.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+
+    def on_predict_begin(self, logs=None):
+        """Called at the beginning of prediction.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
+
+    def on_predict_end(self, logs=None):
+        """Called at the end of prediction.
+
+        Subclasses should override for any actions to run.
+
+        # Arguments
+            logs: dict, currently no data is passed to this argument for this method
+                but that may change in the future.
+        """
 
 
 class BaseLogger(Callback):
@@ -359,7 +634,7 @@ class ModelCheckpoint(Callback):
     """Save the model after every epoch.
 
     `filepath` can contain named formatting options,
-    which will be filled the value of `epoch` and
+    which will be filled with the values of `epoch` and
     keys in `logs` (passed in `on_epoch_end`).
 
     For example: if `filepath` is `weights.{epoch:02d}-{val_loss:.2f}.hdf5`,
@@ -373,6 +648,9 @@ class ModelCheckpoint(Callback):
         save_best_only: if `save_best_only=True`,
             the latest best model according to
             the quantity monitored will not be overwritten.
+        save_weights_only: if True, then only the model's weights will be
+            saved (`model.save_weights(filepath)`), else the full model
+            is saved (`model.save(filepath)`).
         mode: one of {auto, min, max}.
             If `save_best_only=True`, the decision
             to overwrite the current save file is made
@@ -381,9 +659,6 @@ class ModelCheckpoint(Callback):
             this should be `max`, for `val_loss` this should
             be `min`, etc. In `auto` mode, the direction is
             automatically inferred from the name of the monitored quantity.
-        save_weights_only: if True, then only the model's weights will be
-            saved (`model.save_weights(filepath)`), else the full model
-            is saved (`model.save(filepath)`).
         period: Interval (number of epochs) between checkpoints.
     """
 
@@ -477,6 +752,10 @@ class EarlyStopping(Callback):
         baseline: Baseline value for the monitored quantity to reach.
             Training will stop if the model doesn't show improvement
             over the baseline.
+        restore_best_weights: whether to restore model weights from
+            the epoch with the best value of the monitored quantity.
+            If False, the model weights obtained at the last step of
+            training are used.
     """
 
     def __init__(self,
@@ -485,7 +764,8 @@ class EarlyStopping(Callback):
                  patience=0,
                  verbose=0,
                  mode='auto',
-                 baseline=None):
+                 baseline=None,
+                 restore_best_weights=False):
         super(EarlyStopping, self).__init__()
 
         self.monitor = monitor
@@ -495,6 +775,8 @@ class EarlyStopping(Callback):
         self.min_delta = min_delta
         self.wait = 0
         self.stopped_epoch = 0
+        self.restore_best_weights = restore_best_weights
+        self.best_weights = None
 
         if mode not in ['auto', 'min', 'max']:
             warnings.warn('EarlyStopping mode %s is unknown, '
@@ -527,27 +809,40 @@ class EarlyStopping(Callback):
             self.best = np.Inf if self.monitor_op == np.less else -np.Inf
 
     def on_epoch_end(self, epoch, logs=None):
-        current = logs.get(self.monitor)
+        current = self.get_monitor_value(logs)
         if current is None:
-            warnings.warn(
-                'Early stopping conditioned on metric `%s` '
-                'which is not available. Available metrics are: %s' %
-                (self.monitor, ','.join(list(logs.keys()))), RuntimeWarning
-            )
             return
+
         if self.monitor_op(current - self.min_delta, self.best):
             self.best = current
             self.wait = 0
+            if self.restore_best_weights:
+                self.best_weights = self.model.get_weights()
         else:
             self.wait += 1
             if self.wait >= self.patience:
                 self.stopped_epoch = epoch
                 self.model.stop_training = True
+                if self.restore_best_weights:
+                    if self.verbose > 0:
+                        print('Restoring model weights from the end of '
+                              'the best epoch')
+                    self.model.set_weights(self.best_weights)
 
     def on_train_end(self, logs=None):
         if self.stopped_epoch > 0 and self.verbose > 0:
             print('Epoch %05d: early stopping' % (self.stopped_epoch + 1))
 
+    def get_monitor_value(self, logs):
+        monitor_value = logs.get(self.monitor)
+        if monitor_value is None:
+            warnings.warn(
+                'Early stopping conditioned on metric `%s` '
+                'which is not available. Available metrics are: %s' %
+                (self.monitor, ','.join(list(logs.keys()))), RuntimeWarning
+            )
+        return monitor_value
+
 
 class RemoteMonitor(Callback):
     """Callback used to stream events to a server.
@@ -556,16 +851,18 @@ class RemoteMonitor(Callback):
     Events are sent to `root + '/publish/epoch/end/'` by default. Calls are
     HTTP POST, with a `data` argument which is a
     JSON-encoded dictionary of event data.
-    If send_as_json is set to True, the content type of the request will be application/json.
-    Otherwise the serialized JSON will be send within a form
+    If send_as_json is set to True, the content type of the request will be
+    application/json. Otherwise the serialized JSON will be send within a form
 
     # Arguments
         root: String; root url of the target server.
         path: String; path relative to `root` to which the events will be sent.
-        field: String; JSON field under which the data will be stored. The field is used only if the payload is sent
-        within a form (i.e. send_as_json is set to False).
+        field: String; JSON field under which the data will be stored.
+            The field is used only if the payload is sent within a form
+            (i.e. send_as_json is set to False).
         headers: Dictionary; optional custom HTTP headers.
-        send_as_json: Boolean; whether the request should be send as application/json.
+        send_as_json: Boolean; whether the request should be send as
+            application/json.
     """
 
     def __init__(self,
@@ -634,14 +931,18 @@ class LearningRateScheduler(Callback):
                              'should be float.')
         K.set_value(self.model.optimizer.lr, lr)
         if self.verbose > 0:
-            print('\nEpoch %05d: LearningRateScheduler reducing learning '
+            print('\nEpoch %05d: LearningRateScheduler setting learning '
                   'rate to %s.' % (epoch + 1, lr))
 
+    def on_epoch_end(self, epoch, logs=None):
+        logs = logs or {}
+        logs['lr'] = K.get_value(self.model.optimizer.lr)
+
 
 class TensorBoard(Callback):
     """TensorBoard basic visualizations.
 
-    [TensorBoard](https://www.tensorflow.org/get_started/summaries_and_tensorboard)
+    [TensorBoard](https://www.tensorflow.org/guide/summaries_and_tensorboard)
     is a visualization tool provided with TensorFlow.
 
     This callback writes a log for TensorBoard, which allows
@@ -666,13 +967,13 @@ class TensorBoard(Callback):
             and weight histograms for the layers of the model. If set to 0,
             histograms won't be computed. Validation data (or split) must be
             specified for histogram visualizations.
+        batch_size: size of batch of inputs to feed to the network
+            for histograms computation.
         write_graph: whether to visualize the graph in TensorBoard.
             The log file can become quite large when
             write_graph is set to True.
         write_grads: whether to visualize gradient histograms in TensorBoard.
             `histogram_freq` must be greater than 0.
-        batch_size: size of batch of inputs to feed to the network
-            for histograms computation.
         write_images: whether to write model weights to visualize as
             image in TensorBoard.
         embeddings_freq: frequency (in epochs) at which selected embedding
@@ -683,13 +984,20 @@ class TensorBoard(Callback):
             None or empty list all the embedding layer will be watched.
         embeddings_metadata: a dictionary which maps layer name to a file name
             in which metadata for this embedding layer is saved. See the
-            [details](https://www.tensorflow.org/how_tos/embedding_viz/#metadata_optional)
+            [details](https://www.tensorflow.org/guide/embedding#metadata)
             about metadata files format. In case if the same metadata file is
             used for all embedding layers, string can be passed.
         embeddings_data: data to be embedded at layers specified in
             `embeddings_layer_names`. Numpy array (if the model has a single
             input) or list of Numpy arrays (if the model has multiple inputs).
-            Learn [more about embeddings](https://www.tensorflow.org/programmers_guide/embedding)
+            Learn [more about embeddings](
+            https://www.tensorflow.org/guide/embedding).
+        update_freq: `'batch'` or `'epoch'` or integer. When using `'batch'`, writes
+            the losses and metrics to TensorBoard after each batch. The same
+            applies for `'epoch'`. If using an integer, let's say `10000`,
+            the callback will write the metrics and losses to TensorBoard every
+            10000 samples. Note that writing too frequently to TensorBoard
+            can slow down your training.
     """
 
     def __init__(self, log_dir='./logs',
@@ -701,14 +1009,16 @@ class TensorBoard(Callback):
                  embeddings_freq=0,
                  embeddings_layer_names=None,
                  embeddings_metadata=None,
-                 embeddings_data=None):
+                 embeddings_data=None,
+                 update_freq='epoch'):
         super(TensorBoard, self).__init__()
         global tf, projector
         try:
             import tensorflow as tf
             from tensorflow.contrib.tensorboard.plugins import projector
         except ImportError:
-            raise ImportError('You need the TensorFlow module installed to use TensorBoard.')
+            raise ImportError('You need the TensorFlow module installed to '
+                              'use TensorBoard.')
 
         if K.backend() != 'tensorflow':
             if histogram_freq != 0:
@@ -739,6 +1049,13 @@ class TensorBoard(Callback):
         self.embeddings_metadata = embeddings_metadata or {}
         self.batch_size = batch_size
         self.embeddings_data = embeddings_data
+        if update_freq == 'batch':
+            # It is the same as writing as frequently as possible.
+            self.update_freq = 1
+        else:
+            self.update_freq = update_freq
+        self.samples_seen = 0
+        self.samples_seen_at_last_write = 0
 
     def set_model(self, model):
         self.model = model
@@ -746,11 +1063,10 @@ class TensorBoard(Callback):
             self.sess = K.get_session()
         if self.histogram_freq and self.merged is None:
             for layer in self.model.layers:
-
                 for weight in layer.weights:
                     mapped_weight_name = weight.name.replace(':', '_')
                     tf.summary.histogram(mapped_weight_name, weight)
-                    if self.write_grads:
+                    if self.write_grads and weight in layer.trainable_weights:
                         grads = model.optimizer.get_gradients(model.total_loss,
                                                               weight)
 
@@ -759,7 +1075,8 @@ class TensorBoard(Callback):
                         grads = [
                             grad.values if is_indexed_slices(grad) else grad
                             for grad in grads]
-                        tf.summary.histogram('{}_grad'.format(mapped_weight_name), grads)
+                        tf.summary.histogram('{}_grad'.format(mapped_weight_name),
+                                             grads)
                     if self.write_images:
                         w_img = tf.squeeze(weight)
                         shape = K.int_shape(w_img)
@@ -797,7 +1114,8 @@ class TensorBoard(Callback):
                 if hasattr(layer, 'output'):
                     if isinstance(layer.output, list):
                         for i, output in enumerate(layer.output):
-                            tf.summary.histogram('{}_out_{}'.format(layer.name, i), output)
+                            tf.summary.histogram('{}_out_{}'.format(layer.name, i),
+                                                 output)
                     else:
                         tf.summary.histogram('{}_out'.format(layer.name),
                                              layer.output)
@@ -810,7 +1128,8 @@ class TensorBoard(Callback):
             self.writer = tf.summary.FileWriter(self.log_dir)
 
         if self.embeddings_freq and self.embeddings_data is not None:
-            self.embeddings_data = standardize_input_data(self.embeddings_data, model.input_names)
+            self.embeddings_data = standardize_input_data(self.embeddings_data,
+                                                          model.input_names)
 
             embeddings_layer_names = self.embeddings_layer_names
 
@@ -839,8 +1158,6 @@ class TensorBoard(Callback):
 
             self.saver = tf.train.Saver(list(embeddings_vars.values()))
 
-            embeddings_metadata = {}
-
             if not isinstance(self.embeddings_metadata, str):
                 embeddings_metadata = self.embeddings_metadata
             else:
@@ -917,8 +1234,8 @@ class TensorBoard(Callback):
                     batch = slice(i, i + step)
 
                     if type(self.model.input) == list:
-                        feed_dict = {model_input: embeddings_data[idx][batch]
-                                     for idx, model_input in enumerate(self.model.input)}
+                        feed_dict = {_input: embeddings_data[idx][batch]
+                                     for idx, _input in enumerate(self.model.input)}
                     else:
                         feed_dict = {self.model.input: embeddings_data[0][batch]}
 
@@ -929,24 +1246,43 @@ class TensorBoard(Callback):
 
                     self.sess.run(self.assign_embeddings, feed_dict=feed_dict)
                     self.saver.save(self.sess,
-                                    os.path.join(self.log_dir, 'keras_embedding.ckpt'),
+                                    os.path.join(self.log_dir,
+                                                 'keras_embedding.ckpt'),
                                     epoch)
 
                     i += self.batch_size
 
+        if self.update_freq == 'epoch':
+            index = epoch
+        else:
+            index = self.samples_seen
+        self._write_logs(logs, index)
+
+    def _write_logs(self, logs, index):
         for name, value in logs.items():
             if name in ['batch', 'size']:
                 continue
             summary = tf.Summary()
             summary_value = summary.value.add()
-            summary_value.simple_value = value.item()
+            if isinstance(value, np.ndarray):
+                summary_value.simple_value = value.item()
+            else:
+                summary_value.simple_value = value
             summary_value.tag = name
-            self.writer.add_summary(summary, epoch)
+            self.writer.add_summary(summary, index)
         self.writer.flush()
 
     def on_train_end(self, _):
         self.writer.close()
 
+    def on_batch_end(self, batch, logs=None):
+        if self.update_freq != 'epoch':
+            self.samples_seen += logs['size']
+            samples_seen_since = self.samples_seen - self.samples_seen_at_last_write
+            if samples_seen_since >= self.update_freq:
+                self._write_logs(logs, self.samples_seen)
+                self.samples_seen_at_last_write = self.samples_seen
+
 
 class ReduceLROnPlateau(Callback):
     """Reduce learning rate when a metric has stopped improving.
@@ -1060,8 +1396,8 @@ class ReduceLROnPlateau(Callback):
                         new_lr = max(new_lr, self.min_lr)
                         K.set_value(self.model.optimizer.lr, new_lr)
                         if self.verbose > 0:
-                            print('\nEpoch %05d: ReduceLROnPlateau reducing learning '
-                                  'rate to %s.' % (epoch + 1, new_lr))
+                            print('\nEpoch %05d: ReduceLROnPlateau reducing '
+                                  'learning rate to %s.' % (epoch + 1, new_lr))
                         self.cooldown_counter = self.cooldown
                         self.wait = 0
 
@@ -1096,7 +1432,12 @@ class CSVLogger(Callback):
         self.writer = None
         self.keys = None
         self.append_header = True
-        self.file_flags = 'b' if six.PY2 and os.name == 'nt' else ''
+        if six.PY2:
+            self.file_flags = 'b'
+            self._open_args = {}
+        else:
+            self.file_flags = ''
+            self._open_args = {'newline': '\n'}
         super(CSVLogger, self).__init__()
 
     def on_train_begin(self, logs=None):
@@ -1104,9 +1445,12 @@ class CSVLogger(Callback):
             if os.path.exists(self.filename):
                 with open(self.filename, 'r' + self.file_flags) as f:
                     self.append_header = not bool(len(f.readline()))
-            self.csv_file = open(self.filename, 'a' + self.file_flags)
+            mode = 'a'
         else:
-            self.csv_file = open(self.filename, 'w' + self.file_flags)
+            mode = 'w'
+        self.csv_file = io.open(self.filename,
+                                mode + self.file_flags,
+                                **self._open_args)
 
     def on_epoch_end(self, epoch, logs=None):
         logs = logs or {}
@@ -1125,14 +1469,17 @@ class CSVLogger(Callback):
 
         if self.model.stop_training:
             # We set NA so that csv parsers do not fail for this last epoch.
-            logs = dict([(k, logs[k]) if k in logs else (k, 'NA') for k in self.keys])
+            logs = dict([(k, logs[k] if k in logs else 'NA') for k in self.keys])
 
         if not self.writer:
             class CustomDialect(csv.excel):
                 delimiter = self.sep
-
+            fieldnames = ['epoch'] + self.keys
+            if six.PY2:
+                fieldnames = [unicode(x) for x in fieldnames]
             self.writer = csv.DictWriter(self.csv_file,
-                                         fieldnames=['epoch'] + self.keys, dialect=CustomDialect)
+                                         fieldnames=fieldnames,
+                                         dialect=CustomDialect)
             if self.append_header:
                 self.writer.writeheader()
 
