@@ -8,48 +8,54 @@ import numpy as np
 from scipy.sparse import issparse
 
 from .training_utils import batch_shuffle
-from .training_utils import make_batches
 from .training_utils import check_num_samples
+from .training_utils import make_batches
+from .training_utils import should_run_validation
 from .. import backend as K
 from .. import callbacks as cbks
 from ..utils.generic_utils import Progbar
 from ..utils.generic_utils import slice_arrays
+from ..utils.generic_utils import to_list
+from ..utils.generic_utils import unpack_singleton
 
 
-def fit_loop(model, f, ins,
+def fit_loop(model, fit_function, fit_inputs,
              out_labels=None,
              batch_size=None,
              epochs=100,
              verbose=1,
              callbacks=None,
-             val_f=None,
-             val_ins=None,
+             val_function=None,
+             val_inputs=None,
              shuffle=True,
              callback_metrics=None,
              initial_epoch=0,
              steps_per_epoch=None,
-             validation_steps=None):
-    """Abstract fit function for `f(ins)`.
+             validation_steps=None,
+             validation_freq=1):
+    """Abstract fit function for `fit_function(fit_inputs)`.
 
-    Assumes that f returns a list, labeled by out_labels.
+    Assumes that fit_function returns a list, labeled by out_labels.
 
     # Arguments
         model: Keras model instance.
-        f: Keras function returning a list of tensors
-        ins: List of tensors to be fed to `f`
+        fit_function: Keras function returning a list of tensors
+        fit_inputs: List of tensors to be fed to `fit_function`
         out_labels: List of strings, display names of
-            the outputs of `f`
+            the outputs of `fit_function`
         batch_size: Integer batch size or None if unknown.
         epochs: Number of times to iterate over the data
         verbose: Verbosity mode, 0, 1 or 2
-        callbacks: List of callbacks to be called during training
-        val_f: Keras function to call for validation
-        val_ins: List of tensors to be fed to `val_f`
+        callbacks: List of callbacks to be called during training and validation
+            (if `val_function` and `val_inputs` are not `None`).
+        val_function: Keras function to call for validation
+        val_inputs: List of tensors to be fed to `val_function`
         shuffle: Whether to shuffle the data at the beginning of each epoch
         callback_metrics: List of strings, the display names of the metrics
             passed to the callbacks. They should be the
             concatenation of list the display names of the outputs of
-             `f` and the list of display names of the outputs of `f_val`.
+             `fit_function` and the list of display names
+             of the outputs of `fit_inputs`.
         initial_epoch: Epoch at which to start training
             (useful for resuming a previous training run)
         steps_per_epoch: Total number of steps (batches of samples)
@@ -58,17 +64,24 @@ def fit_loop(model, f, ins,
         validation_steps: Number of steps to run validation for
             (only if doing validation from data tensors).
             Ignored with the default value of `None`.
+        validation_freq: Only relevant if validation data is provided. Integer
+            or list/tuple/set. If an integer, specifies how many training
+            epochs to run before a new validation run is performed, e.g.
+            validation_freq=2` runs validation every 2 epochs. If a list,
+            tuple, or set, specifies the epochs on which to run validation,
+            e.g. `validation_freq=[1, 2, 10]` runs validation at the end
+            of the 1st, 2nd, and 10th epochs.
 
     # Returns
         `History` object.
     """
     do_validation = False
-    if val_f and val_ins:
+    if val_function and val_inputs:
         do_validation = True
-        if (verbose and ins and
-           hasattr(ins[0], 'shape') and hasattr(val_ins[0], 'shape')):
+        if (verbose and fit_inputs and
+           hasattr(fit_inputs[0], 'shape') and hasattr(val_inputs[0], 'shape')):
             print('Train on %d samples, validate on %d samples' %
-                  (ins[0].shape[0], val_ins[0].shape[0]))
+                  (fit_inputs[0].shape[0], val_inputs[0].shape[0]))
     if validation_steps:
         do_validation = True
         if steps_per_epoch is None:
@@ -82,7 +95,7 @@ def fit_loop(model, f, ins,
                              'to perform validation '
                              'when doing step-wise training.')
 
-    num_train_samples = check_num_samples(ins,
+    num_train_samples = check_num_samples(fit_inputs,
                                           batch_size=batch_size,
                                           steps=steps_per_epoch,
                                           steps_name='steps_per_epoch')
@@ -107,10 +120,7 @@ def fit_loop(model, f, ins,
 
     # it's possible to callback a different model than itself
     # (used by Sequential models)
-    if hasattr(model, 'callback_model') and model.callback_model:
-        callback_model = model.callback_model
-    else:
-        callback_model = model
+    callback_model = model._get_callback_model()
 
     callbacks.set_model(callback_model)
     callbacks.set_params({
@@ -122,10 +132,10 @@ def fit_loop(model, f, ins,
         'do_validation': do_validation,
         'metrics': callback_metrics or [],
     })
-    callbacks.on_train_begin()
-    callback_model.stop_training = False
+    callbacks._call_begin_hook('train')
+    callbacks.model.stop_training = False
     for cbk in callbacks:
-        cbk.validation_data = val_ins
+        cbk.validation_data = val_inputs
 
     # To prevent a slowdown,
     # we find beforehand the arrays that need conversion.
@@ -134,7 +144,7 @@ def fit_loop(model, f, ins,
             model._feed_sample_weights)
     indices_for_conversion_to_dense = []
     for i in range(len(feed)):
-        if issparse(ins[i]) and not K.is_sparse(feed[i]):
+        if issparse(fit_inputs[i]) and not K.is_sparse(feed[i]):
             indices_for_conversion_to_dense.append(i)
 
     for epoch in range(initial_epoch, epochs):
@@ -145,27 +155,24 @@ def fit_loop(model, f, ins,
         epoch_logs = {}
         if steps_per_epoch is not None:
             for step_index in range(steps_per_epoch):
-                batch_logs = {}
-                batch_logs['batch'] = step_index
-                batch_logs['size'] = 1
-                callbacks.on_batch_begin(step_index, batch_logs)
-                outs = f(ins)
+                batch_logs = {'batch': step_index, 'size': 1}
+                callbacks._call_batch_hook('train', 'begin', step_index, batch_logs)
+                outs = fit_function(fit_inputs)
 
-                if not isinstance(outs, list):
-                    outs = [outs]
+                outs = to_list(outs)
                 for l, o in zip(out_labels, outs):
                     batch_logs[l] = o
 
-                callbacks.on_batch_end(step_index, batch_logs)
+                callbacks._call_batch_hook('train', 'end', step_index, batch_logs)
                 if callback_model.stop_training:
                     break
 
-            if do_validation:
-                val_outs = test_loop(model, val_f, val_ins,
+            if do_validation and should_run_validation(validation_freq, epoch):
+                val_outs = test_loop(model, val_function, val_inputs,
                                      steps=validation_steps,
+                                     callbacks=callbacks,
                                      verbose=0)
-                if not isinstance(val_outs, list):
-                    val_outs = [val_outs]
+                val_outs = to_list(val_outs)
                 # Same labels assumed.
                 for l, o in zip(out_labels, val_outs):
                     epoch_logs['val_' + l] = o
@@ -179,51 +186,53 @@ def fit_loop(model, f, ins,
             for batch_index, (batch_start, batch_end) in enumerate(batches):
                 batch_ids = index_array[batch_start:batch_end]
                 try:
-                    if isinstance(ins[-1], float):
+                    if isinstance(fit_inputs[-1], float):
                         # Do not slice the training phase flag.
                         ins_batch = slice_arrays(
-                            ins[:-1], batch_ids) + [ins[-1]]
+                            fit_inputs[:-1], batch_ids) + [fit_inputs[-1]]
                     else:
-                        ins_batch = slice_arrays(ins, batch_ids)
+                        ins_batch = slice_arrays(fit_inputs, batch_ids)
                 except TypeError:
                     raise TypeError('TypeError while preparing batch. '
                                     'If using HDF5 input data, '
                                     'pass shuffle="batch".')
-                batch_logs = {}
-                batch_logs['batch'] = batch_index
-                batch_logs['size'] = len(batch_ids)
-                callbacks.on_batch_begin(batch_index, batch_logs)
+                batch_logs = {'batch': batch_index, 'size': len(batch_ids)}
+                callbacks._call_batch_hook('train', 'begin', batch_index, batch_logs)
                 for i in indices_for_conversion_to_dense:
                     ins_batch[i] = ins_batch[i].toarray()
 
-                outs = f(ins_batch)
-                if not isinstance(outs, list):
-                    outs = [outs]
+                outs = fit_function(ins_batch)
+                outs = to_list(outs)
                 for l, o in zip(out_labels, outs):
                     batch_logs[l] = o
 
-                callbacks.on_batch_end(batch_index, batch_logs)
-                if callback_model.stop_training:
+                callbacks._call_batch_hook('train', 'end', batch_index, batch_logs)
+                if callbacks.model.stop_training:
                     break
 
-                if batch_index == len(batches) - 1:  # Last batch.
-                    if do_validation:
-                        val_outs = test_loop(model, val_f, val_ins,
-                                             batch_size=batch_size,
-                                             verbose=0)
-                        if not isinstance(val_outs, list):
-                            val_outs = [val_outs]
-                        # Same labels assumed.
-                        for l, o in zip(out_labels, val_outs):
-                            epoch_logs['val_' + l] = o
+            if batch_index == len(batches) - 1:  # Last batch.
+                if do_validation and should_run_validation(validation_freq, epoch):
+                    val_outs = test_loop(model, val_function, val_inputs,
+                                         batch_size=batch_size,
+                                         callbacks=callbacks,
+                                         verbose=0)
+                    val_outs = to_list(val_outs)
+                    # Same labels assumed.
+                    for l, o in zip(out_labels, val_outs):
+                        epoch_logs['val_' + l] = o
+
         callbacks.on_epoch_end(epoch, epoch_logs)
-        if callback_model.stop_training:
+        if callbacks.model.stop_training:
             break
-    callbacks.on_train_end()
+    callbacks._call_end_hook('train')
     return model.history
 
 
-def predict_loop(model, f, ins, batch_size=32, verbose=0, steps=None):
+def predict_loop(model, f, ins,
+                 batch_size=32,
+                 verbose=0,
+                 steps=None,
+                 callbacks=None):
     """Abstract method to loop over some data in batches.
 
     # Arguments
@@ -235,6 +244,8 @@ def predict_loop(model, f, ins, batch_size=32, verbose=0, steps=None):
         steps: Total number of steps (batches of samples)
             before declaring `predict_loop` finished.
             Ignored with the default value of `None`.
+        callbacks: List of callbacks or an instance of
+            `keras.callbacks.CallbackList` to be called during prediction.
 
     # Returns
         Array of predictions (if the model has a single output)
@@ -245,6 +256,20 @@ def predict_loop(model, f, ins, batch_size=32, verbose=0, steps=None):
                                     batch_size=batch_size,
                                     steps=steps,
                                     steps_name='steps')
+
+    # Check if callbacks have not been already configured
+    if not isinstance(callbacks, cbks.CallbackList):
+        callbacks = cbks.CallbackList(callbacks)
+        callback_model = model._get_callback_model()
+        callbacks.set_model(callback_model)
+        callback_params = {
+            'batch_size': batch_size,
+            'steps': steps,
+            'samples': num_samples,
+            'verbose': verbose,
+        }
+        callbacks.set_params(callback_params)
+
     if verbose == 1:
         if steps is not None:
             progbar = Progbar(target=steps)
@@ -256,6 +281,9 @@ def predict_loop(model, f, ins, batch_size=32, verbose=0, steps=None):
         if issparse(ins[i]) and not K.is_sparse(model._feed_inputs[i]):
             indices_for_conversion_to_dense.append(i)
 
+    callbacks.model.stop_training = False
+    callbacks._call_begin_hook('predict')
+
     if steps is not None:
         # Step-based predictions.
         # Since we do not know how many samples
@@ -265,16 +293,21 @@ def predict_loop(model, f, ins, batch_size=32, verbose=0, steps=None):
         # and concatenate them upon returning.
         unconcatenated_outs = []
         for step in range(steps):
+            batch_logs = {'batch': step, 'size': 1}
+            callbacks._call_batch_hook('predict', 'begin', step, batch_logs)
             batch_outs = f(ins)
-            if not isinstance(batch_outs, list):
-                batch_outs = [batch_outs]
+            batch_outs = to_list(batch_outs)
             if step == 0:
                 for batch_out in batch_outs:
                     unconcatenated_outs.append([])
             for i, batch_out in enumerate(batch_outs):
                 unconcatenated_outs[i].append(batch_out)
+
+            batch_logs['outputs'] = batch_outs
+            callbacks._call_batch_hook('predict', 'end', step, batch_logs)
             if verbose == 1:
                 progbar.update(step + 1)
+        callbacks.on_predict_end()
         if len(unconcatenated_outs) == 1:
             return np.concatenate(unconcatenated_outs[0], axis=0)
         return [np.concatenate(unconcatenated_outs[i], axis=0)
@@ -294,9 +327,10 @@ def predict_loop(model, f, ins, batch_size=32, verbose=0, steps=None):
             for i in indices_for_conversion_to_dense:
                 ins_batch[i] = ins_batch[i].toarray()
 
+            batch_logs = {'batch': batch_index, 'size': len(batch_ids)}
+            callbacks._call_batch_hook('predict', 'begin', batch_index, batch_logs)
             batch_outs = f(ins_batch)
-            if not isinstance(batch_outs, list):
-                batch_outs = [batch_outs]
+            batch_outs = to_list(batch_outs)
             if batch_index == 0:
                 # Pre-allocate the results arrays.
                 for batch_out in batch_outs:
@@ -304,14 +338,20 @@ def predict_loop(model, f, ins, batch_size=32, verbose=0, steps=None):
                     outs.append(np.zeros(shape, dtype=batch_out.dtype))
             for i, batch_out in enumerate(batch_outs):
                 outs[i][batch_start:batch_end] = batch_out
+
+            batch_logs['outputs'] = batch_outs
+            callbacks._call_batch_hook('predict', 'end', batch_index, batch_logs)
             if verbose == 1:
                 progbar.update(batch_end)
-        if len(outs) == 1:
-            return outs[0]
-        return outs
+        callbacks._call_end_hook('predict')
+        return unpack_singleton(outs)
 
 
-def test_loop(model, f, ins, batch_size=None, verbose=0, steps=None):
+def test_loop(model, f, ins,
+              batch_size=None,
+              verbose=0,
+              steps=None,
+              callbacks=None):
     """Abstract method to loop over some data in batches.
 
     # Arguments
@@ -323,6 +363,8 @@ def test_loop(model, f, ins, batch_size=None, verbose=0, steps=None):
         steps: Total number of steps (batches of samples)
             before declaring predictions finished.
             Ignored with the default value of `None`.
+        callbacks: List of callbacks or an instance of
+            `keras.callbacks.CallbackList` to be called during evaluation.
 
     # Returns
         Scalar loss (if the model has a single output and no metrics)
@@ -344,6 +386,24 @@ def test_loop(model, f, ins, batch_size=None, verbose=0, steps=None):
                                     batch_size=batch_size,
                                     steps=steps,
                                     steps_name='steps')
+
+    # Check if callbacks have not been already configured
+    if not isinstance(callbacks, cbks.CallbackList):
+        callbacks = cbks.CallbackList(callbacks)
+        callback_model = model._get_callback_model()
+        callbacks.set_model(callback_model)
+        callback_metrics = []
+        if hasattr(model, 'metrics_names'):
+            callback_metrics = list(model.metrics_names)
+        callback_params = {
+            'batch_size': batch_size,
+            'steps': steps,
+            'samples': num_samples,
+            'verbose': verbose,
+            'metrics': callback_metrics,
+        }
+        callbacks.set_params(callback_params)
+
     outs = []
     if verbose == 1:
         if steps is not None:
@@ -361,13 +421,17 @@ def test_loop(model, f, ins, batch_size=None, verbose=0, steps=None):
         if issparse(ins[i]) and not K.is_sparse(feed[i]):
             indices_for_conversion_to_dense.append(i)
 
+    callbacks.model.stop_training = False
+    callbacks._call_begin_hook('test')
+
     if steps is not None:
         for step in range(steps):
+            batch_logs = {'batch': step, 'size': 1}
+            callbacks._call_batch_hook('test', 'begin', step, batch_logs)
             batch_outs = f(ins)
             if isinstance(batch_outs, list):
                 if step == 0:
-                    for _ in enumerate(batch_outs):
-                        outs.append(0.)
+                    outs.extend([0.] * len(batch_outs))
                 for i, batch_out in enumerate(batch_outs):
                     if i in stateful_metric_indices:
                         outs[i] = float(batch_out)
@@ -377,6 +441,12 @@ def test_loop(model, f, ins, batch_size=None, verbose=0, steps=None):
                 if step == 0:
                     outs.append(0.)
                 outs[0] += batch_outs
+
+            if hasattr(model, 'metrics_names'):
+                for l, o in zip(model.metrics_names, batch_outs):
+                    batch_logs[l] = o
+            callbacks._call_batch_hook('test', 'end', step, batch_logs)
+
             if verbose == 1:
                 progbar.update(step + 1)
         for i in range(len(outs)):
@@ -395,11 +465,12 @@ def test_loop(model, f, ins, batch_size=None, verbose=0, steps=None):
             for i in indices_for_conversion_to_dense:
                 ins_batch[i] = ins_batch[i].toarray()
 
+            batch_logs = {'batch': batch_index, 'size': len(batch_ids)}
+            callbacks._call_batch_hook('test', 'begin', batch_index, batch_logs)
             batch_outs = f(ins_batch)
             if isinstance(batch_outs, list):
                 if batch_index == 0:
-                    for batch_out in enumerate(batch_outs):
-                        outs.append(0.)
+                    outs.extend([0.] * len(batch_outs))
                 for i, batch_out in enumerate(batch_outs):
                     if i in stateful_metric_indices:
                         outs[i] = batch_out
@@ -410,11 +481,15 @@ def test_loop(model, f, ins, batch_size=None, verbose=0, steps=None):
                     outs.append(0.)
                 outs[0] += batch_outs * len(batch_ids)
 
+            if hasattr(model, 'metrics_names'):
+                for l, o in zip(model.metrics_names, batch_outs):
+                    batch_logs[l] = o
+            callbacks._call_batch_hook('test', 'end', batch_index, batch_logs)
+
             if verbose == 1:
                 progbar.update(batch_end)
         for i in range(len(outs)):
             if i not in stateful_metric_indices:
                 outs[i] /= num_samples
-    if len(outs) == 1:
-        return outs[0]
-    return outs
+    callbacks._call_end_hook('test')
+    return unpack_singleton(outs)
