@@ -7,11 +7,16 @@ from __future__ import print_function
 import warnings
 import numpy as np
 
+from .training_utils import is_sequence
+from .training_utils import iter_sequence_infinite
+from .training_utils import should_run_validation
 from .. import backend as K
 from ..utils.data_utils import Sequence
 from ..utils.data_utils import GeneratorEnqueuer
 from ..utils.data_utils import OrderedEnqueuer
 from ..utils.generic_utils import Progbar
+from ..utils.generic_utils import to_list
+from ..utils.generic_utils import unpack_singleton
 from .. import callbacks as cbks
 
 
@@ -23,6 +28,7 @@ def fit_generator(model,
                   callbacks=None,
                   validation_data=None,
                   validation_steps=None,
+                  validation_freq=1,
                   class_weight=None,
                   max_queue_size=10,
                   workers=1,
@@ -30,7 +36,6 @@ def fit_generator(model,
                   shuffle=True,
                   initial_epoch=0):
     """See docstring for `Model.fit_generator`."""
-    wait_time = 0.01  # in seconds
     epoch = initial_epoch
 
     do_validation = bool(validation_data)
@@ -38,15 +43,15 @@ def fit_generator(model,
     if do_validation:
         model._make_test_function()
 
-    is_sequence = isinstance(generator, Sequence)
-    if not is_sequence and use_multiprocessing and workers > 1:
+    use_sequence_api = is_sequence(generator)
+    if not use_sequence_api and use_multiprocessing and workers > 1:
         warnings.warn(
             UserWarning('Using a generator with `use_multiprocessing=True`'
                         ' and multiple workers may duplicate your data.'
-                        ' Please consider using the`keras.utils.Sequence'
+                        ' Please consider using the `keras.utils.Sequence'
                         ' class.'))
     if steps_per_epoch is None:
-        if is_sequence:
+        if use_sequence_api:
             steps_per_epoch = len(generator)
         else:
             raise ValueError('`steps_per_epoch=None` is only valid for a'
@@ -57,10 +62,11 @@ def fit_generator(model,
 
     # python 2 has 'next', 3 has '__next__'
     # avoid any explicit version checks
+    val_use_sequence_api = is_sequence(validation_data)
     val_gen = (hasattr(validation_data, 'next') or
                hasattr(validation_data, '__next__') or
-               isinstance(validation_data, Sequence))
-    if (val_gen and not isinstance(validation_data, Sequence) and
+               val_use_sequence_api)
+    if (val_gen and not val_use_sequence_api and
             not validation_steps):
         raise ValueError('`validation_steps=None` is only valid for a'
                          ' generator based on the `keras.utils.Sequence`'
@@ -84,10 +90,8 @@ def fit_generator(model,
     callbacks = cbks.CallbackList(_callbacks)
 
     # it's possible to callback a different model than self:
-    if hasattr(model, 'callback_model') and model.callback_model:
-        callback_model = model.callback_model
-    else:
-        callback_model = model
+    callback_model = model._get_callback_model()
+
     callbacks.set_model(callback_model)
     callbacks.set_params({
         'epochs': epochs,
@@ -96,35 +100,58 @@ def fit_generator(model,
         'do_validation': do_validation,
         'metrics': callback_metrics,
     })
-    callbacks.on_train_begin()
+    callbacks._call_begin_hook('train')
 
     enqueuer = None
     val_enqueuer = None
 
     try:
-        if do_validation and not val_gen:
-            # Prepare data for validation
-            if len(validation_data) == 2:
-                val_x, val_y = validation_data
-                val_sample_weight = None
-            elif len(validation_data) == 3:
-                val_x, val_y, val_sample_weight = validation_data
+        if do_validation:
+            if val_gen and workers > 0:
+                # Create an Enqueuer that can be reused
+                val_data = validation_data
+                if is_sequence(val_data):
+                    val_enqueuer = OrderedEnqueuer(
+                        val_data,
+                        use_multiprocessing=use_multiprocessing)
+                    validation_steps = validation_steps or len(val_data)
+                else:
+                    val_enqueuer = GeneratorEnqueuer(
+                        val_data,
+                        use_multiprocessing=use_multiprocessing)
+                val_enqueuer.start(workers=workers,
+                                   max_queue_size=max_queue_size)
+                val_enqueuer_gen = val_enqueuer.get()
+            elif val_gen:
+                val_data = validation_data
+                if is_sequence(val_data):
+                    val_enqueuer_gen = iter_sequence_infinite(val_data)
+                    validation_steps = validation_steps or len(val_data)
+                else:
+                    val_enqueuer_gen = val_data
             else:
-                raise ValueError('`validation_data` should be a tuple '
-                                 '`(val_x, val_y, val_sample_weight)` '
-                                 'or `(val_x, val_y)`. Found: ' +
-                                 str(validation_data))
-            val_x, val_y, val_sample_weights = model._standardize_user_data(
-                val_x, val_y, val_sample_weight)
-            val_data = val_x + val_y + val_sample_weights
-            if model.uses_learning_phase and not isinstance(K.learning_phase(),
-                                                            int):
-                val_data += [0.]
-            for cbk in callbacks:
-                cbk.validation_data = val_data
+                # Prepare data for validation
+                if len(validation_data) == 2:
+                    val_x, val_y = validation_data
+                    val_sample_weight = None
+                elif len(validation_data) == 3:
+                    val_x, val_y, val_sample_weight = validation_data
+                else:
+                    raise ValueError('`validation_data` should be a tuple '
+                                     '`(val_x, val_y, val_sample_weight)` '
+                                     'or `(val_x, val_y)`. Found: ' +
+                                     str(validation_data))
+                val_x, val_y, val_sample_weights = model._standardize_user_data(
+                    val_x, val_y, val_sample_weight)
+                val_data = val_x + val_y + val_sample_weights
+                if model.uses_learning_phase and not isinstance(K.learning_phase(),
+                                                                int):
+                    val_data += [0.]
+                for cbk in callbacks:
+                    cbk.validation_data = val_data
 
         if workers > 0:
-            if is_sequence:
+            if use_sequence_api:
                 enqueuer = OrderedEnqueuer(
                     generator,
                     use_multiprocessing=use_multiprocessing,
@@ -132,17 +159,16 @@ def fit_generator(model,
             else:
                 enqueuer = GeneratorEnqueuer(
                     generator,
-                    use_multiprocessing=use_multiprocessing,
-                    wait_time=wait_time)
+                    use_multiprocessing=use_multiprocessing)
             enqueuer.start(workers=workers, max_queue_size=max_queue_size)
             output_generator = enqueuer.get()
         else:
-            if is_sequence:
-                output_generator = iter(generator)
+            if use_sequence_api:
+                output_generator = iter_sequence_infinite(generator)
             else:
                 output_generator = generator
 
-        callback_model.stop_training = False
+        callbacks.model.stop_training = False
         # Construct epoch logs.
         epoch_logs = {}
         while epoch < epochs:
@@ -170,8 +196,6 @@ def fit_generator(model,
                                      'a tuple `(x, y, sample_weight)` '
                                      'or `(x, y)`. Found: ' +
                                      str(generator_output))
-                # build batch logs
-                batch_logs = {}
                 if x is None or len(x) == 0:
                     # Handle data tensors support when no input given
                     # step-size = 1 for data tensors
@@ -182,33 +206,35 @@ def fit_generator(model,
                     batch_size = list(x.values())[0].shape[0]
                 else:
                     batch_size = x.shape[0]
-                batch_logs['batch'] = batch_index
-                batch_logs['size'] = batch_size
+                # build batch logs
+                batch_logs = {'batch': batch_index, 'size': batch_size}
                 callbacks.on_batch_begin(batch_index, batch_logs)
 
                 outs = model.train_on_batch(x, y,
                                             sample_weight=sample_weight,
                                             class_weight=class_weight)
 
-                if not isinstance(outs, list):
-                    outs = [outs]
+                outs = to_list(outs)
                 for l, o in zip(out_labels, outs):
                     batch_logs[l] = o
 
-                callbacks.on_batch_end(batch_index, batch_logs)
+                callbacks._call_batch_hook('train', 'end', batch_index, batch_logs)
 
                 batch_index += 1
                 steps_done += 1
 
                 # Epoch finished.
-                if steps_done >= steps_per_epoch and do_validation:
+                if (steps_done >= steps_per_epoch and
+                        do_validation and
+                        should_run_validation(validation_freq, epoch)):
+                    # Note that `callbacks` here is an instance of
+                    # `keras.callbacks.CallbackList`
                     if val_gen:
                         val_outs = model.evaluate_generator(
-                            validation_data,
+                            val_enqueuer_gen,
                             validation_steps,
-                            workers=workers,
-                            use_multiprocessing=use_multiprocessing,
-                            max_queue_size=max_queue_size)
+                            callbacks=callbacks,
+                            workers=0)
                     else:
                         # No need for try/except because
                         # data has already been validated.
@@ -216,19 +242,19 @@ def fit_generator(model,
                             val_x, val_y,
                             batch_size=batch_size,
                             sample_weight=val_sample_weights,
+                            callbacks=callbacks,
                             verbose=0)
-                    if not isinstance(val_outs, list):
-                        val_outs = [val_outs]
+                    val_outs = to_list(val_outs)
                     # Same labels assumed.
                     for l, o in zip(out_labels, val_outs):
                         epoch_logs['val_' + l] = o
 
-                if callback_model.stop_training:
+                if callbacks.model.stop_training:
                     break
 
             callbacks.on_epoch_end(epoch, epoch_logs)
             epoch += 1
-            if callback_model.stop_training:
+            if callbacks.model.stop_training:
                 break
 
     finally:
@@ -239,12 +265,13 @@ def fit_generator(model,
             if val_enqueuer is not None:
                 val_enqueuer.stop()
 
-    callbacks.on_train_end()
+    callbacks._call_end_hook('train')
     return model.history
 
 
 def evaluate_generator(model, generator,
                        steps=None,
+                       callbacks=None,
                        max_queue_size=10,
                        workers=1,
                        use_multiprocessing=False,
@@ -252,7 +279,6 @@ def evaluate_generator(model, generator,
     """See docstring for `Model.evaluate_generator`."""
     model._make_test_function()
 
-    stateful_metric_indices = []
     if hasattr(model, 'metrics'):
         for m in model.stateful_metric_functions:
             m.reset_states()
@@ -263,18 +289,17 @@ def evaluate_generator(model, generator,
         stateful_metric_indices = []
 
     steps_done = 0
-    wait_time = 0.01
     outs_per_batch = []
     batch_sizes = []
-    is_sequence = isinstance(generator, Sequence)
-    if not is_sequence and use_multiprocessing and workers > 1:
+    use_sequence_api = is_sequence(generator)
+    if not use_sequence_api and use_multiprocessing and workers > 1:
         warnings.warn(
             UserWarning('Using a generator with `use_multiprocessing=True`'
                         ' and multiple workers may duplicate your data.'
-                        ' Please consider using the`keras.utils.Sequence'
+                        ' Please consider using the `keras.utils.Sequence'
                         ' class.'))
     if steps is None:
-        if is_sequence:
+        if use_sequence_api:
             steps = len(generator)
         else:
             raise ValueError('`steps=None` is only valid for a generator'
@@ -283,22 +308,39 @@ def evaluate_generator(model, generator,
                              ' `keras.utils.Sequence` class.')
     enqueuer = None
 
+    # Check if callbacks have not been already configured
+    if not isinstance(callbacks, cbks.CallbackList):
+        callbacks = cbks.CallbackList(callbacks)
+        callback_model = model._get_callback_model()
+        callbacks.set_model(callback_model)
+        callback_metrics = []
+        if hasattr(model, 'metrics_names'):
+            callback_metrics = list(model.metrics_names)
+        callback_params = {
+            'steps': steps,
+            'verbose': verbose,
+            'metrics': callback_metrics,
+        }
+        callbacks.set_params(callback_params)
+
+    callbacks.model.stop_training = False
+    callbacks._call_begin_hook('test')
+
     try:
         if workers > 0:
-            if is_sequence:
+            if use_sequence_api:
                 enqueuer = OrderedEnqueuer(
                     generator,
                     use_multiprocessing=use_multiprocessing)
             else:
                 enqueuer = GeneratorEnqueuer(
                     generator,
-                    use_multiprocessing=use_multiprocessing,
-                    wait_time=wait_time)
+                    use_multiprocessing=use_multiprocessing)
             enqueuer.start(workers=workers, max_queue_size=max_queue_size)
             output_generator = enqueuer.get()
         else:
-            if is_sequence:
-                output_generator = iter(generator)
+            if use_sequence_api:
+                output_generator = iter_sequence_infinite(generator)
             else:
                 output_generator = generator
 
@@ -322,10 +364,6 @@ def evaluate_generator(model, generator,
                                  '(x, y, sample_weight) '
                                  'or (x, y). Found: ' +
                                  str(generator_output))
-            outs = model.test_on_batch(x, y, sample_weight=sample_weight)
-            if not isinstance(outs, list):
-                outs = [outs]
-            outs_per_batch.append(outs)
 
             if x is None or len(x) == 0:
                 # Handle data tensors support when no input given
@@ -341,10 +379,24 @@ def evaluate_generator(model, generator,
                 raise ValueError('Received an empty batch. '
                                  'Batches should contain '
                                  'at least one item.')
+
+            batch_logs = {'batch': steps_done, 'size': batch_size}
+            callbacks._call_batch_hook('test', 'begin', steps_done, batch_logs)
+            outs = model.test_on_batch(x, y, sample_weight=sample_weight)
+            outs = to_list(outs)
+            outs_per_batch.append(outs)
+
+            if hasattr(model, 'metrics_names'):
+                for l, o in zip(model.metrics_names, outs):
+                    batch_logs[l] = o
+            callbacks._call_batch_hook('test', 'end', steps_done, batch_logs)
+
             steps_done += 1
             batch_sizes.append(batch_size)
+
             if verbose == 1:
                 progbar.update(steps_done)
+        callbacks._call_end_hook('test')
 
     finally:
         if enqueuer is not None:
@@ -356,14 +408,13 @@ def evaluate_generator(model, generator,
             averages.append(np.average([out[i] for out in outs_per_batch],
                                        weights=batch_sizes))
         else:
-            averages.append(float(outs_per_batch[-1][i]))
-    if len(averages) == 1:
-        return averages[0]
-    return averages
+            averages.append(np.float64(outs_per_batch[-1][i]))
+    return unpack_singleton(averages)
 
 
 def predict_generator(model, generator,
                       steps=None,
+                      callbacks=None,
                       max_queue_size=10,
                       workers=1,
                       use_multiprocessing=False,
@@ -372,17 +423,16 @@ def predict_generator(model, generator,
     model._make_predict_function()
 
     steps_done = 0
-    wait_time = 0.01
     all_outs = []
-    is_sequence = isinstance(generator, Sequence)
-    if not is_sequence and use_multiprocessing and workers > 1:
+    use_sequence_api = is_sequence(generator)
+    if not use_sequence_api and use_multiprocessing and workers > 1:
         warnings.warn(
             UserWarning('Using a generator with `use_multiprocessing=True`'
                         ' and multiple workers may duplicate your data.'
-                        ' Please consider using the`keras.utils.Sequence'
+                        ' Please consider using the `keras.utils.Sequence'
                         ' class.'))
     if steps is None:
-        if is_sequence:
+        if use_sequence_api:
             steps = len(generator)
         else:
             raise ValueError('`steps=None` is only valid for a generator'
@@ -391,22 +441,35 @@ def predict_generator(model, generator,
                              ' `keras.utils.Sequence` class.')
     enqueuer = None
 
+    # Check if callbacks have not been already configured
+    if not isinstance(callbacks, cbks.CallbackList):
+        callbacks = cbks.CallbackList(callbacks)
+        callback_model = model._get_callback_model()
+        callbacks.set_model(callback_model)
+        callback_params = {
+            'steps': steps,
+            'verbose': verbose,
+        }
+        callbacks.set_params(callback_params)
+
+    callbacks.model.stop_training = False
+    callbacks._call_begin_hook('predict')
+
     try:
         if workers > 0:
-            if is_sequence:
+            if use_sequence_api:
                 enqueuer = OrderedEnqueuer(
                     generator,
                     use_multiprocessing=use_multiprocessing)
             else:
                 enqueuer = GeneratorEnqueuer(
                     generator,
-                    use_multiprocessing=use_multiprocessing,
-                    wait_time=wait_time)
+                    use_multiprocessing=use_multiprocessing)
             enqueuer.start(workers=workers, max_queue_size=max_queue_size)
             output_generator = enqueuer.get()
         else:
-            if is_sequence:
-                output_generator = iter(generator)
+            if use_sequence_api:
+                output_generator = iter_sequence_infinite(generator)
             else:
                 output_generator = generator
 
@@ -432,9 +495,26 @@ def predict_generator(model, generator,
                 # yields inputs (not targets and sample weights).
                 x = generator_output
 
+            if x is None or len(x) == 0:
+                # Handle data tensors support when no input given
+                # step-size = 1 for data tensors
+                batch_size = 1
+            elif isinstance(x, list):
+                batch_size = x[0].shape[0]
+            elif isinstance(x, dict):
+                batch_size = list(x.values())[0].shape[0]
+            else:
+                batch_size = x.shape[0]
+            if batch_size == 0:
+                raise ValueError('Received an empty batch. '
+                                 'Batches should contain '
+                                 'at least one item.')
+
+            batch_logs = {'batch': steps_done, 'size': batch_size}
+            callbacks._call_batch_hook('predict', 'begin', steps_done, batch_logs)
+
             outs = model.predict_on_batch(x)
-            if not isinstance(outs, list):
-                outs = [outs]
+            outs = to_list(outs)
 
             if not all_outs:
                 for out in outs:
@@ -442,10 +522,14 @@ def predict_generator(model, generator,
 
             for i, out in enumerate(outs):
                 all_outs[i].append(out)
+
+            batch_logs['outputs'] = outs
+            callbacks._call_batch_hook('predict', 'end', steps_done, batch_logs)
+
             steps_done += 1
             if verbose == 1:
                 progbar.update(steps_done)
-
+        callbacks._call_end_hook('predict')
     finally:
         if enqueuer is not None:
             enqueuer.stop()
