@@ -124,7 +124,7 @@ class TruncatedNormal(Initializer):
 
     These values are similar to values from a `RandomNormal`
     except that values more than two standard deviations from the mean
-    are discarded and re-drawn. This is the recommended initializer for
+    are discarded and redrawn. This is the recommended initializer for
     neural network weights and filters.
 
     # Arguments
@@ -234,7 +234,8 @@ class Orthogonal(Initializer):
         seed: A Python integer. Used to seed the random generator.
 
     # References
-        Saxe et al., http://arxiv.org/abs/1312.6120
+        - [Exact solutions to the nonlinear dynamics of learning in deep
+           linear neural networks](http://arxiv.org/abs/1312.6120)
     """
 
     def __init__(self, gain=1., seed=None):
@@ -247,9 +248,10 @@ class Orthogonal(Initializer):
             num_rows *= dim
         num_cols = shape[-1]
         flat_shape = (num_rows, num_cols)
+        rng = np.random
         if self.seed is not None:
-            np.random.seed(self.seed)
-        a = np.random.normal(0.0, 1.0, flat_shape)
+            rng = np.random.RandomState(self.seed)
+        a = rng.normal(0.0, 1.0, flat_shape)
         u, _, v = np.linalg.svd(a, full_matrices=False)
         # Pick the one with the correct shape.
         q = u if u.shape == flat_shape else v
@@ -266,7 +268,9 @@ class Orthogonal(Initializer):
 class Identity(Initializer):
     """Initializer that generates the identity matrix.
 
-    Only use for square 2D matrices.
+    Only use for 2D matrices.
+    If the desired matrix is not square, it pads with zeros on the
+    additional rows/columns
 
     # Arguments
         gain: Multiplicative factor to apply to the identity matrix.
@@ -276,11 +280,11 @@ class Identity(Initializer):
         self.gain = gain
 
     def __call__(self, shape, dtype=None):
-        if len(shape) != 2 or shape[0] != shape[1]:
-            raise ValueError('Identity matrix initializer can only be used '
-                             'for 2D square matrices.')
-        else:
-            return self.gain * np.identity(shape[0])
+        if len(shape) != 2:
+            raise ValueError(
+                'Identity matrix initializer can only be used for 2D matrices.')
+
+        return self.gain * K.eye((shape[0], shape[1]), dtype=dtype)
 
     def get_config(self):
         return {
@@ -302,8 +306,7 @@ def lecun_uniform(seed=None):
         An initializer.
 
     # References
-        LeCun 98, Efficient Backprop,
-        http://yann.lecun.com/exdb/publis/pdf/lecun-98b.pdf
+        - [Efficient BackProp](http://yann.lecun.com/exdb/publis/pdf/lecun-98b.pdf)
     """
     return VarianceScaling(scale=1.,
                            mode='fan_in',
@@ -326,8 +329,8 @@ def glorot_normal(seed=None):
         An initializer.
 
     # References
-        Glorot & Bengio, AISTATS 2010
-        http://jmlr.org/proceedings/papers/v9/glorot10a/glorot10a.pdf
+        - [Understanding the difficulty of training deep feedforward neural
+           networks](http://jmlr.org/proceedings/papers/v9/glorot10a/glorot10a.pdf)
     """
     return VarianceScaling(scale=1.,
                            mode='fan_avg',
@@ -350,8 +353,8 @@ def glorot_uniform(seed=None):
         An initializer.
 
     # References
-        Glorot & Bengio, AISTATS 2010
-        http://jmlr.org/proceedings/papers/v9/glorot10a/glorot10a.pdf
+        - [Understanding the difficulty of training deep feedforward neural
+           networks](http://jmlr.org/proceedings/papers/v9/glorot10a/glorot10a.pdf)
     """
     return VarianceScaling(scale=1.,
                            mode='fan_avg',
@@ -373,7 +376,8 @@ def he_normal(seed=None):
         An initializer.
 
     # References
-        He et al., http://arxiv.org/abs/1502.01852
+        - [Delving Deep into Rectifiers: Surpassing Human-Level Performance on
+           ImageNet Classification](http://arxiv.org/abs/1502.01852)
     """
     return VarianceScaling(scale=2.,
                            mode='fan_in',
@@ -418,7 +422,8 @@ def he_uniform(seed=None):
         An initializer.
 
     # References
-        He et al., http://arxiv.org/abs/1502.01852
+        - [Delving Deep into Rectifiers: Surpassing Human-Level Performance on
+           ImageNet Classification](http://arxiv.org/abs/1502.01852)
     """
     return VarianceScaling(scale=2.,
                            mode='fan_in',
