@@ -6,10 +6,10 @@ from __future__ import division
 from __future__ import print_function
 
 import copy
-from ..engine import Layer
-from ..engine import InputSpec
-from ..engine.topology import _object_list_uid
+from ..engine.base_layer import Layer
+from ..engine.base_layer import InputSpec
 from ..utils.generic_utils import has_arg
+from ..utils.generic_utils import object_list_uid
 from .. import backend as K
 
 from . import recurrent
@@ -71,7 +71,7 @@ class Wrapper(Layer):
         # get the updates from the inner layer.
         inner_inputs = inputs
         if inputs is not None:
-            uid = _object_list_uid(inputs)
+            uid = object_list_uid(inputs)
             if uid in self._input_map:
                 inner_inputs = self._input_map[uid]
 
@@ -160,6 +160,37 @@ class TimeDistributed(Wrapper):
         super(TimeDistributed, self).__init__(layer, **kwargs)
         self.supports_masking = True
 
+    def _get_shape_tuple(self, init_tuple, tensor, start_idx, int_shape=None):
+        """Finds non-specific dimensions in the static shapes
+        and replaces them by the corresponding dynamic shapes of the tensor.
+
+        # Arguments
+            init_tuple: a tuple, the first part of the output shape
+            tensor: the tensor from which to get the (static and dynamic) shapes
+                as the last part of the output shape
+            start_idx: int, which indicate the first dimension to take from
+                the static shape of the tensor
+            int_shape: an alternative static shape to take as the last part
+                of the output shape
+
+        # Returns
+            The new int_shape with the first part from init_tuple
+            and the last part from either `int_shape` (if provided)
+            or K.int_shape(tensor), where every `None` is replaced by
+            the corresponding dimension from K.shape(tensor)
+        """
+        # replace all None in int_shape by K.shape
+        if int_shape is None:
+            int_shape = K.int_shape(tensor)[start_idx:]
+        if not any(not s for s in int_shape):
+            return init_tuple + int_shape
+        tensor_shape = K.shape(tensor)
+        int_shape = list(int_shape)
+        for i, s in enumerate(int_shape):
+            if not s:
+                int_shape[i] = tensor_shape[start_idx + i]
+        return init_tuple + tuple(int_shape)
+
     def build(self, input_shape):
         assert len(input_shape) >= 3
         self.input_spec = InputSpec(shape=input_shape)
@@ -204,18 +235,24 @@ class TimeDistributed(Wrapper):
             input_length = input_shape[1]
             if not input_length:
                 input_length = K.shape(inputs)[1]
+            inner_input_shape = self._get_shape_tuple((-1,), inputs, 2)
             # Shape: (num_samples * timesteps, ...). And track the
             # transformation in self._input_map.
-            input_uid = _object_list_uid(inputs)
-            inputs = K.reshape(inputs, (-1,) + input_shape[2:])
+            input_uid = object_list_uid(inputs)
+            inputs = K.reshape(inputs, inner_input_shape)
             self._input_map[input_uid] = inputs
             # (num_samples * timesteps, ...)
+            if has_arg(self.layer.call, 'mask') and mask is not None:
+                inner_mask_shape = self._get_shape_tuple((-1,), mask, 2)
+                kwargs['mask'] = K.reshape(mask, inner_mask_shape)
             y = self.layer.call(inputs, **kwargs)
             if hasattr(y, '_uses_learning_phase'):
                 uses_learning_phase = y._uses_learning_phase
             # Shape: (num_samples, timesteps, ...)
             output_shape = self.compute_output_shape(input_shape)
-            y = K.reshape(y, (-1, input_length) + output_shape[2:])
+            output_shape = self._get_shape_tuple(
+                (-1, input_length), y, 1, output_shape[2:])
+            y = K.reshape(y, output_shape)
 
         # Apply activity regularizer if any:
         if (hasattr(self.layer, 'activity_regularizer') and
@@ -227,6 +264,71 @@ class TimeDistributed(Wrapper):
             y._uses_learning_phase = True
         return y
 
+    def compute_mask(self, inputs, mask=None):
+        """Computes an output mask tensor for Embedding layer
+        based on the inputs, mask, and the inner layer.
+
+        If batch size is specified:
+        Simply return the input `mask`. (An rnn-based implementation with
+        more than one rnn inputs is required but not supported in Keras yet.)
+
+        Otherwise we call `compute_mask` of the inner layer at each time step.
+        If the output mask at each time step is not `None`:
+        (E.g., inner layer is Masking or RNN)
+        Concatenate all of them and return the concatenation.
+        If the output mask at each time step is `None` and
+        the input mask is not `None`:
+        (E.g., inner layer is Dense)
+        Reduce the input_mask to 2 dimensions and return it.
+        Otherwise (both the output mask and the input mask are `None`):
+        (E.g., `mask` is not used at all)
+        Return `None`.
+
+        # Arguments
+            inputs: Tensor
+            mask: Tensor
+        # Returns
+            None or a tensor
+        """
+        # cases need to call the layer.compute_mask when input_mask is None:
+        # Masking layer and Embedding layer with mask_zero
+        input_shape = K.int_shape(inputs)
+        if input_shape[0]:
+            # batch size matters, we currently do not handle mask explicitly
+            return mask
+        inner_mask = mask
+        if inner_mask is not None:
+            inner_mask_shape = self._get_shape_tuple((-1,), mask, 2)
+            inner_mask = K.reshape(inner_mask, inner_mask_shape)
+        input_uid = object_list_uid(inputs)
+        inner_inputs = self._input_map[input_uid]
+        output_mask = self.layer.compute_mask(inner_inputs, inner_mask)
+        if output_mask is None:
+            if mask is None:
+                return None
+            # input_mask is not None, and output_mask is None:
+            # we should return a not-None mask
+            output_mask = mask
+            for _ in range(2, len(K.int_shape(mask))):
+                output_mask = K.any(output_mask, axis=-1)
+        else:
+            # output_mask is not None. We need to reshape it
+            input_length = input_shape[1]
+            if not input_length:
+                input_length = K.shape(inputs)[1]
+            output_mask_int_shape = K.int_shape(output_mask)
+            if output_mask_int_shape is None:
+                # if the output_mask does not have a static shape,
+                # its shape must be the same as mask's
+                if mask is not None:
+                    output_mask_int_shape = K.int_shape(mask)
+                else:
+                    output_mask_int_shape = K.compute_output_shape(input_shape)[:-1]
+            output_mask_shape = self._get_shape_tuple(
+                (-1, input_length), output_mask, 1, output_mask_int_shape[1:])
+            output_mask = K.reshape(output_mask, output_mask_shape)
+        return output_mask
+
 
 class Bidirectional(Wrapper):
     """Bidirectional wrapper for RNNs.
@@ -238,6 +340,7 @@ class Bidirectional(Wrapper):
             One of {'sum', 'mul', 'concat', 'ave', None}.
             If None, the outputs will not be combined,
             they will be returned as a list.
+        weights: Initial weights to load in the Bidirectional model
 
     # Raises
         ValueError: In case of invalid `merge_mode` argument.
@@ -374,6 +477,10 @@ class Bidirectional(Wrapper):
             # Perform the call with temporarily replaced input_spec
             original_input_spec = self.input_spec
             self.input_spec = full_input_spec
+            if 'initial_state' in kwargs:
+                kwargs.pop('initial_state')
+            if 'constants' in kwargs:
+                kwargs.pop('constants')
             output = super(Bidirectional, self).__call__(full_input, **kwargs)
             self.input_spec = original_input_spec
             return output
@@ -392,14 +499,31 @@ class Bidirectional(Wrapper):
         if has_arg(self.layer.call, 'mask'):
             kwargs['mask'] = mask
         if has_arg(self.layer.call, 'constants'):
+            if self._num_constants is not None and constants is None:
+                    constants = inputs[-self._num_constants:]
+                    inputs = inputs[:-self._num_constants]
             kwargs['constants'] = constants
-
-        if initial_state is not None and has_arg(self.layer.call, 'initial_state'):
-            forward_state = initial_state[:len(initial_state) // 2]
-            backward_state = initial_state[len(initial_state) // 2:]
-            y = self.forward_layer.call(inputs, initial_state=forward_state, **kwargs)
-            y_rev = self.backward_layer.call(inputs, initial_state=backward_state, **kwargs)
+        if has_arg(self.layer.call, 'initial_state'):
+            if isinstance(inputs, list) and len(inputs) > 1:
+                if initial_state is not None:
+                    raise ValueError('Layer was passed initial state ' +
+                                     'via both kwarg and inputs list)')
+                initial_state = inputs[1:]
+                inputs = [inputs[0]]
+            if initial_state is None:
+                forward_state = None
+                backward_state = None
+            else:
+                pivot = len(initial_state) // 2
+                forward_state = initial_state[:pivot]
+                backward_state = initial_state[pivot:]
+            y = self.forward_layer.call(inputs,
+                                        initial_state=forward_state, **kwargs)
+            y_rev = self.backward_layer.call(inputs,
+                                             initial_state=backward_state, **kwargs)
         else:
+            if isinstance(inputs, list) and len(inputs) > 1 or initial_state:
+                raise ValueError('Layer does not accept initial_state argument.')
             y = self.forward_layer.call(inputs, **kwargs)
             y_rev = self.backward_layer.call(inputs, **kwargs)
 
@@ -420,6 +544,9 @@ class Bidirectional(Wrapper):
             output = y * y_rev
         elif self.merge_mode is None:
             output = [y, y_rev]
+        else:
+            raise ValueError('Unrecognized value for argument '
+                             'merge_mode: %s' % (self.merge_mode))
 
         # Properly set learning phase
         if (getattr(y, '_uses_learning_phase', False) or
@@ -487,12 +614,24 @@ class Bidirectional(Wrapper):
             return self.forward_layer.updates + self.backward_layer.updates
         return []
 
+    def get_updates_for(self, inputs=None):
+        forward_updates = self.forward_layer.get_updates_for(inputs)
+        backward_updates = self.backward_layer.get_updates_for(inputs)
+        return (super(Wrapper, self).get_updates_for(inputs) +
+                forward_updates + backward_updates)
+
     @property
     def losses(self):
         if hasattr(self.forward_layer, 'losses'):
             return self.forward_layer.losses + self.backward_layer.losses
         return []
 
+    def get_losses_for(self, inputs=None):
+        forward_losses = self.forward_layer.get_losses_for(inputs)
+        backward_losses = self.backward_layer.get_losses_for(inputs)
+        return (super(Wrapper, self).get_losses_for(inputs) +
+                forward_losses + backward_losses)
+
     @property
     def constraints(self):
         constraints = {}
