@@ -7,6 +7,7 @@ from . import conv_utils
 
 # Globally-importable utils.
 from .io_utils import HDF5Matrix
+from .io_utils import H5Dict
 from .data_utils import get_file
 from .data_utils import Sequence
 from .data_utils import GeneratorEnqueuer
@@ -18,7 +19,9 @@ from .generic_utils import serialize_keras_object
 from .generic_utils import deserialize_keras_object
 from .generic_utils import Progbar
 from .layer_utils import convert_all_kernels_in_model
+from .layer_utils import get_source_inputs
 from .layer_utils import print_summary
+from .vis_utils import model_to_dot
 from .vis_utils import plot_model
 from .np_utils import to_categorical
 from .np_utils import normalize
