@@ -9,6 +9,7 @@ from .. import backend as K
 from ..layers.core import Lambda
 from ..engine.training import Model
 from ..models import clone_model
+from ..utils.generic_utils import to_list
 
 
 def _get_available_devices():
@@ -58,7 +59,9 @@ def multi_gpu_model(model, gpus=None, cpu_merge=True, cpu_relocation=False):
         A Keras `Model` instance which can be used just like the initial
         `model` argument, but which distributes its workload on multiple GPUs.
 
-    # Example 1 - Training models with weights merge on CPU
+    # Examples
+
+    Example 1 - Training models with weights merge on CPU
 
     ```python
         import tensorflow as tf
@@ -99,7 +102,7 @@ def multi_gpu_model(model, gpus=None, cpu_merge=True, cpu_relocation=False):
         model.save('my_model.h5')
     ```
 
-    # Example 2 - Training models with weights merge on CPU using cpu_relocation
+    Example 2 - Training models with weights merge on CPU using cpu_relocation
 
     ```python
          ..
@@ -107,16 +110,16 @@ def multi_gpu_model(model, gpus=None, cpu_merge=True, cpu_relocation=False):
          model = Xception(weights=None, ..)
 
          try:
-             model = multi_gpu_model(model, cpu_relocation=True)
+             parallel_model = multi_gpu_model(model, cpu_relocation=True)
              print("Training using multiple GPUs..")
-         except:
+         except ValueError:
+             parallel_model = model
              print("Training using single GPU or CPU..")
-
-         model.compile(..)
+         parallel_model.compile(..)
          ..
     ```
 
-    # Example 3 - Training models with weights merge on GPU (recommended for NV-link)
+    Example 3 - Training models with weights merge on GPU (recommended for NV-link)
 
     ```python
          ..
@@ -124,12 +127,13 @@ def multi_gpu_model(model, gpus=None, cpu_merge=True, cpu_relocation=False):
          model = Xception(weights=None, ..)
 
          try:
-             model = multi_gpu_model(model, cpu_merge=False)
+             parallel_model = multi_gpu_model(model, cpu_merge=False)
              print("Training using multiple GPUs..")
          except:
+             parallel_model = model
              print("Training using single GPU or CPU..")
 
-         model.compile(..)
+         parallel_model.compile(..)
          ..
     ```
 
@@ -149,7 +153,7 @@ def multi_gpu_model(model, gpus=None, cpu_merge=True, cpu_relocation=False):
     if not gpus:
         # Using all visible GPUs when not specifying `gpus`
         # e.g. CUDA_VISIBLE_DEVICES=0,2 python keras_mgpu.py
-        gpus = len([x for x in available_devices if 'gpu' in x])
+        gpus = len([x for x in available_devices if '/gpu:' in x])
 
     if isinstance(gpus, (list, tuple)):
         if len(gpus) <= 1:
@@ -224,17 +228,31 @@ def multi_gpu_model(model, gpus=None, cpu_merge=True, cpu_relocation=False):
                 # Apply model on slice
                 # (creating a model replica on the target device).
                 outputs = model(inputs)
-                if not isinstance(outputs, list):
-                    outputs = [outputs]
+                outputs = to_list(outputs)
 
                 # Save the outputs for merging back together later.
                 for o in range(len(outputs)):
                     all_outputs[o].append(outputs[o])
 
+    # Deduplicate output names to handle Siamese networks.
+    occurrences = {}
+    for n in model.output_names:
+        if n not in occurrences:
+            occurrences[n] = 1
+        else:
+            occurrences[n] += 1
+    conflict_counter = {n: 0 for n, count in occurrences.items() if count > 1}
+    output_names = []
+    for n in model.output_names:
+        if n in conflict_counter:
+            conflict_counter[n] += 1
+            n += '_%d' % conflict_counter[n]
+        output_names.append(n)
+
     # Merge outputs under expected scope.
     with tf.device('/cpu:0' if cpu_merge else '/gpu:%d' % target_gpu_ids[0]):
         merged = []
-        for name, outputs in zip(model.output_names, all_outputs):
+        for name, outputs in zip(output_names, all_outputs):
             merged.append(concatenate(outputs,
                                       axis=0, name=name))
         return Model(model.inputs, merged)
