@@ -4,13 +4,12 @@ from numpy.testing import assert_allclose
 
 import keras
 from keras.utils.test_utils import layer_test
-from keras.utils.test_utils import keras_test
 from keras.layers import recurrent
 from keras.layers import embeddings
 from keras.models import Sequential
 from keras.models import Model
-from keras.engine.topology import Input
-from keras.layers.core import Masking
+from keras.engine import Input
+from keras.layers import Masking
 from keras import regularizers
 from keras import backend as K
 
@@ -18,28 +17,16 @@ num_samples, timesteps, embedding_dim, units = 2, 5, 4, 3
 embedding_num = 12
 
 
-@keras_test
-def rnn_test(f):
-    """
-    All the recurrent layers share the same interface,
-    so we can run through them with a single function.
-    """
-    f = keras_test(f)
-    return pytest.mark.parametrize('layer_class', [
-        recurrent.SimpleRNN,
-        recurrent.GRU,
-        recurrent.LSTM
-    ])(f)
+rnn_test = pytest.mark.parametrize('layer_class',
+                                   [recurrent.SimpleRNN,
+                                    recurrent.GRU,
+                                    recurrent.LSTM])
 
 
-@keras_test
-def rnn_cell_test(f):
-    f = keras_test(f)
-    return pytest.mark.parametrize('cell_class', [
-        recurrent.SimpleRNNCell,
-        recurrent.GRUCell,
-        recurrent.LSTMCell
-    ])(f)
+rnn_cell_test = pytest.mark.parametrize('cell_class',
+                                        [recurrent.SimpleRNNCell,
+                                         recurrent.GRUCell,
+                                         recurrent.LSTMCell])
 
 
 @rnn_test
@@ -178,6 +165,120 @@ def test_masking_correctness(layer_class):
     assert_allclose(out7, out6, atol=1e-5)
 
 
+@pytest.mark.skipif(K.backend() == 'cntk', reason='Not supported.')
+def test_masking_correctness_output_not_equal_to_first_state():
+
+    class Cell(keras.layers.Layer):
+
+        def __init__(self):
+            self.state_size = None
+            self.output_size = None
+            super(Cell, self).__init__()
+
+        def build(self, input_shape):
+            self.state_size = input_shape[-1]
+            self.output_size = input_shape[-1]
+
+        def call(self, inputs, states):
+            return inputs, [s + 1 for s in states]
+
+    num_samples = 5
+    num_timesteps = 4
+    state_size = input_size = 3  # also equal to `output_size`
+
+    # random inputs and state values
+    x_vals = np.random.random((num_samples, num_timesteps, input_size))
+    # last timestep masked for first sample (all zero inputs masked by Masking layer)
+    x_vals[0, -1, :] = 0
+    s_initial_vals = np.random.random((num_samples, state_size))
+
+    # final outputs equal to last inputs
+    y_vals_expected = x_vals[:, -1].copy()
+    # except for first sample, where it is equal to second to last value due to mask
+    y_vals_expected[0] = x_vals[0, -2]
+
+    s_final_vals_expected = s_initial_vals.copy()
+    # states are incremented `num_timesteps - 1` times for first sample
+    s_final_vals_expected[0] += (num_timesteps - 1)
+    # and `num_timesteps - 1` times for remaining samples
+    s_final_vals_expected[1:] += num_timesteps
+
+    for unroll in [True, False]:
+        x = Input((num_timesteps, input_size), name="x")
+        x_masked = Masking()(x)
+        s_initial = Input((state_size,), name="s_initial")
+        y, s_final = recurrent.RNN(Cell(),
+                                   return_state=True,
+                                   unroll=unroll)(x_masked, initial_state=s_initial)
+        model = Model([x, s_initial], [y, s_final])
+        model.compile(optimizer='sgd', loss='mse')
+
+        y_vals, s_final_vals = model.predict([x_vals, s_initial_vals])
+        assert_allclose(y_vals,
+                        y_vals_expected,
+                        err_msg="Unexpected output for unroll={}".format(unroll))
+        assert_allclose(s_final_vals,
+                        s_final_vals_expected,
+                        err_msg="Unexpected state for unroll={}".format(unroll))
+
+
+@pytest.mark.skipif(K.backend() == 'cntk', reason='Not supported.')
+def test_masking_correctness_output_size_not_equal_to_first_state_size():
+
+    class Cell(keras.layers.Layer):
+
+        def __init__(self):
+            self.state_size = None
+            self.output_size = None
+            super(Cell, self).__init__()
+
+        def build(self, input_shape):
+            self.state_size = input_shape[-1]
+            self.output_size = input_shape[-1] * 2
+
+        def call(self, inputs, states):
+            return keras.layers.concatenate([inputs] * 2), [s + 1 for s in states]
+
+    num_samples = 5
+    num_timesteps = 6
+    input_size = state_size = 7
+
+    # random inputs and state values
+    x_vals = np.random.random((num_samples, num_timesteps, input_size))
+    # last timestep masked for first sample (all zero inputs masked by Masking layer)
+    x_vals[0, -1, :] = 0
+    s_initial_vals = np.random.random((num_samples, state_size))
+
+    # final outputs equal to last inputs concatenated
+    y_vals_expected = np.concatenate([x_vals[:, -1]] * 2, axis=-1)
+    # except for first sample, where it is equal to second to last value due to mask
+    y_vals_expected[0] = np.concatenate([x_vals[0, -2]] * 2, axis=-1)
+
+    s_final_vals_expected = s_initial_vals.copy()
+    # states are incremented `num_timesteps - 1` times for first sample
+    s_final_vals_expected[0] += (num_timesteps - 1)
+    # and `num_timesteps - 1` times for remaining samples
+    s_final_vals_expected[1:] += num_timesteps
+
+    for unroll in [True, False]:
+        x = Input((num_timesteps, input_size), name="x")
+        x_masked = Masking()(x)
+        s_initial = Input((state_size,), name="s_initial")
+        y, s_final = recurrent.RNN(Cell(),
+                                   return_state=True,
+                                   unroll=unroll)(x_masked, initial_state=s_initial)
+        model = Model([x, s_initial], [y, s_final])
+        model.compile(optimizer='sgd', loss='mse')
+
+        y_vals, s_final_vals = model.predict([x_vals, s_initial_vals])
+        assert_allclose(y_vals,
+                        y_vals_expected,
+                        err_msg="Unexpected output for unroll={}".format(unroll))
+        assert_allclose(s_final_vals,
+                        s_final_vals_expected,
+                        err_msg="Unexpected state for unroll={}".format(unroll))
+
+
 @rnn_test
 def test_implementation_mode(layer_class):
     for mode in [1, 2]:
@@ -239,7 +340,6 @@ def test_trainability(layer_class):
     assert len(layer.non_trainable_weights) == 0
 
 
-@keras_test
 def test_masking_layer():
     ''' This test based on a previously failing issue here:
     https://github.com/keras-team/keras/issues/1567
@@ -430,7 +530,6 @@ def test_state_reuse_with_dropout(layer_class):
     outputs = model.predict(inputs)
 
 
-@keras_test
 def test_minimal_rnn_cell_non_layer():
 
     class MinimalRNNCell(object):
@@ -466,7 +565,6 @@ def test_minimal_rnn_cell_non_layer():
     model.train_on_batch(np.zeros((6, 5, 5)), np.zeros((6, 32)))
 
 
-@keras_test
 def test_minimal_rnn_cell_non_layer_multiple_states():
 
     class MinimalRNNCell(object):
@@ -499,14 +597,13 @@ def test_minimal_rnn_cell_non_layer_multiple_states():
              MinimalRNNCell(16, 8),
              MinimalRNNCell(32, 16)]
     layer = recurrent.RNN(cells)
-    assert layer.cell.state_size == (32, 32, 16, 16, 8, 8)
+    assert layer.cell.state_size == (8, 8, 16, 16, 32, 32)
     y = layer(x)
     model = keras.models.Model(x, y)
     model.compile(optimizer='rmsprop', loss='mse')
     model.train_on_batch(np.zeros((6, 5, 5)), np.zeros((6, 32)))
 
 
-@keras_test
 def test_minimal_rnn_cell_layer():
 
     class MinimalRNNCell(keras.layers.Layer):
@@ -632,7 +729,6 @@ def test_builtin_rnn_cell_layer(cell_class):
     assert_allclose(y_np, y_np_2, atol=1e-4)
 
 
-@keras_test
 @pytest.mark.skipif((K.backend() in ['cntk', 'theano']),
                     reason='Not supported.')
 def test_stacked_rnn_dropout():
@@ -649,7 +745,6 @@ def test_stacked_rnn_dropout():
     model.train_on_batch(x_np, y_np)
 
 
-@keras_test
 def test_stacked_rnn_attributes():
     cells = [recurrent.LSTMCell(3),
              recurrent.LSTMCell(3, kernel_regularizer='l2')]
@@ -672,12 +767,24 @@ def test_stacked_rnn_attributes():
     assert layer.get_losses_for(x) == [y]
 
 
-@keras_test
 def test_stacked_rnn_compute_output_shape():
     cells = [recurrent.LSTMCell(3),
              recurrent.LSTMCell(6)]
     layer = recurrent.RNN(cells, return_state=True, return_sequences=True)
     output_shape = layer.compute_output_shape((None, timesteps, embedding_dim))
+    expected_output_shape = [(None, timesteps, 6),
+                             (None, 3),
+                             (None, 3),
+                             (None, 6),
+                             (None, 6)]
+    assert output_shape == expected_output_shape
+
+    # Test reverse_state_order = True for stacked cell.
+    stacked_cell = recurrent.StackedRNNCells(
+        cells, reverse_state_order=True)
+    layer = recurrent.RNN(
+        stacked_cell, return_state=True, return_sequences=True)
+    output_shape = layer.compute_output_shape((None, timesteps, embedding_dim))
     expected_output_shape = [(None, timesteps, 6),
                              (None, 6),
                              (None, 6),
@@ -698,7 +805,6 @@ def test_batch_size_equal_one(layer_class):
     model.train_on_batch(x, y)
 
 
-@keras_test
 def test_rnn_cell_with_constants_layer():
 
     class RNNCellWithConstants(keras.layers.Layer):
@@ -807,7 +913,6 @@ def test_rnn_cell_with_constants_layer():
     assert_allclose(y_np, y_np_2, atol=1e-4)
 
 
-@keras_test
 def test_rnn_cell_with_constants_layer_passing_initial_state():
 
     class RNNCellWithConstants(keras.layers.Layer):
@@ -896,5 +1001,58 @@ def test_rnn_cell_with_constants_layer_passing_initial_state():
     assert_allclose(y_np, y_np_3, atol=1e-4)
 
 
+@rnn_test
+def test_rnn_cell_identity_initializer(layer_class):
+    inputs = Input(shape=(timesteps, embedding_dim))
+    layer = layer_class(units, recurrent_initializer='identity')
+    layer(inputs)
+    recurrent_kernel = layer.get_weights()[1]
+    num_kernels = recurrent_kernel.shape[1] // recurrent_kernel.shape[0]
+    assert np.array_equal(recurrent_kernel,
+                          np.concatenate([np.identity(units)] * num_kernels, axis=1))
+
+
+@pytest.mark.skipif(K.backend() == 'cntk', reason='Not supported.')
+def test_inconsistent_output_state_size():
+
+    class PlusOneRNNCell(keras.layers.Layer):
+        """Add one to the input and state.
+
+        This cell is used for testing state_size and output_size."""
+
+        def __init__(self, num_unit, **kwargs):
+            self.state_size = num_unit
+            super(PlusOneRNNCell, self).__init__(**kwargs)
+
+        def build(self, input_shape):
+            self.output_size = input_shape[-1]
+
+        def call(self, inputs, states):
+            return inputs + 1, [states[0] + 1]
+
+    batch = 32
+    time_step = 4
+    state_size = 5
+    input_size = 6
+    cell = PlusOneRNNCell(state_size)
+    x = keras.Input((None, input_size))
+    layer = recurrent.RNN(cell)
+    y = layer(x)
+
+    assert cell.state_size == state_size
+    init_state = layer.get_initial_state(x)
+    assert len(init_state) == 1
+    if K.backend() != 'theano':
+        # theano does not support static shape inference.
+        assert K.int_shape(init_state[0]) == (None, state_size)
+
+    model = keras.models.Model(x, y)
+    model.compile(optimizer='rmsprop', loss='mse')
+    model.train_on_batch(
+        np.zeros((batch, time_step, input_size)),
+        np.zeros((batch, input_size)))
+    assert model.output_shape == (None, input_size)
+
+
 if __name__ == '__main__':
     pytest.main([__file__])
